@@ -6,6 +6,7 @@ Base classes for interfacing with the arborist service for authz. Please use
 
 import inspect
 import json
+import datetime
 from collections import deque
 from urllib.parse import quote
 
@@ -193,9 +194,11 @@ class BaseArboristClient(AuthzClient):
                         for n in backoff.fibo():
                             yield n / 2.0
 
-                    await backoff.on_predicate(wait_gen, on_giveup=giveup, **retry)(
+                    res = backoff.on_predicate(wait_gen, on_giveup=giveup, **retry)(
                         self.healthy
                     )()
+                    if inspect.isawaitable(res):
+                        res = await res
                     rv = await client.request(method, url, **kwargs)
                 else:
                     raise
@@ -225,7 +228,7 @@ class BaseArboristClient(AuthzClient):
     async def get_user(self, username):
         url = "{}/{}".format(self._user_url, quote(username))
         response = await self.get(url)
-        if response.code != 204:
+        if response.code != 200:
             raise ArboristError(response.error_msg, response.code)
         return response.json
 
@@ -270,8 +273,24 @@ class BaseArboristClient(AuthzClient):
         return response.json
 
     @maybe_sync
-    async def auth_request(self, jwt, service, methods, resources):
+    async def auth_request(self, jwt, service, methods, resources, user_id=None):
         """
+        Args:
+            jwt (str):
+                valid jwt access token
+            service (str):
+                service name the user is checking access to (ex. fence, indexd)
+            methods (list/str):
+                Identifier for the action the user is trying to do. Like ``resource``, this
+                is something that has to exist in arborist already.
+            resources (list/str):
+                List of identifiers for the thing being accessed. These look like filepaths. This
+                ``resources`` must correspond to some resource entered previously in
+                arborist.
+            user_id (str):
+                User's user id that corresponds to the user id in the arborist usr table.
+                NOTE: user_id overrides any jwt token provided
+
         Return:
             bool: authorization response
         """
@@ -280,13 +299,18 @@ class BaseArboristClient(AuthzClient):
         if isinstance(methods, string_types):
             methods = [methods]
         data = {
-            "user": {"token": jwt},
             "requests": [
                 {"resource": resource, "action": {"service": service, "method": method}}
                 for resource in resources
                 for method in methods
             ],
         }
+
+        if user_id:
+            data["user"] = {"user_id": user_id}
+        elif jwt:
+            data["user"] = {"token": jwt}
+
         response = await self.post(self._auth_url.rstrip("/") + "/request", json=data)
         if not response.successful:
             msg = "request to arborist failed: {}".format(response.error_msg)
@@ -657,6 +681,53 @@ class BaseArboristClient(AuthzClient):
         elif response.code != 201:
             self.logger.error(response.error_msg)
 
+    @maybe_sync
+    async def update_user(self, username, new_username=None, new_email=None):
+        """
+        Update an existing user's username and/or email. At least one of
+        new_username, new_email must be passed a value that is not None.
+
+        Args:
+            username (str): username of user to be updated
+            new_username (str): new username to update user with
+            new_email (str): new email to update user with
+
+        Return:
+            ArboristResponse: Arborist response
+
+        Raises:
+            - ValueError: if this function is called incorrectly
+            - ArboristError: if the operation failed (couldn't update user)
+        """
+        url = "{}/{}".format(self._user_url, quote(username))
+        request = {}
+        if new_username is not None:
+            request["name"] = new_username
+        if new_email is not None:
+            request["email"] = new_email
+        if len(request) == 0:
+            raise ValueError(
+                "update_user requires that at least one of new_username, "
+                "new_email be passed a value that is not None"
+            )
+        if username == new_username:
+            raise ValueError(
+                "update_user requires that new_username not be equal to username"
+            )
+        response = await self.patch(url, json=request, expect_json=False)
+        if response.code != 204:
+            msg = "could not update Arborist user `{}` with request body `{}`. status code: {}. error message: {}".format(
+                username, request, response.code, response.error_msg
+            )
+            self.logger.error(msg)
+            raise ArboristError(msg, response.code)
+        self.logger.info(
+            "successfully updated Arborist user `{}` with request body `{}`".format(
+                username, request
+            )
+        )
+        return response
+
     @maybe_sync
     async def list_resources_for_user(self, username):
         """
@@ -673,12 +744,20 @@ class BaseArboristClient(AuthzClient):
         return response.json["resources"]
 
     @maybe_sync
-    async def grant_user_policy(self, username, policy_id):
+    async def grant_user_policy(self, username, policy_id, expires_at=None):
         """
-        MUST be user name, and not serial user ID
+        Grant a policy to a user
+
+        Args:
+            username (str): MUST be user's username, and not serial user ID
+            policy_id (str): Arborist policy id
+            expires_at (int): POSIX timestamp for when the policy should expire
         """
         url = self._user_url + "/{}/policy".format(quote(username))
         request = {"policy": policy_id}
+        if expires_at is not None:
+            expires_at = datetime.datetime.utcfromtimestamp(expires_at)
+            request["expires_at"] = expires_at.isoformat() + "Z"
         response = await self.post(url, json=request, expect_json=False)
         if response.code != 204:
             self.logger.error(
